#!/usr/bin/python

import re, sys

def normalize_input(input):
    """
    Normalize an input string:
    1. %BEGIN and END% are on their own lines (if not in line-comment)
    2. Each maximal term starts on a new line.
    (Terms can still cross lines.)
    Bug: periods in quoted strings with linebreaks cause linebreaks.
    """
    result = []
    for line in input.splitlines(True):  # keep newlines

        # separate line comment (ignoring %BEGIN, END%), if any

        x = re.split('(?<!END)%(?!BEGIN)', line, 1)
        work = x[0]
        comment = '%' + x[1] if len(x) == 2 else ''

        # UGLY: don't split on periods in double quotes.
        # This does not handle quotes strings with linebreaks!
        # Get strings, inside quotes, with periods.
        pats = re.findall('(?<=")[^"]*\.[^"](?=")', work)
        # In those strings, replace periods with temporary character (").
        pairs = []  # save list of changes so we can undo them.
        for pat in pats:
            rep = re.sub('\.', '"', pat)
            work = re.sub(pat, rep, work)
            pairs.append((pat,rep))

        # put %BEGIN and END% on their own lines

        work = re.sub('\s*(?<!^)%BEGIN', '\n%BEGIN', work)
        work = re.sub('%BEGIN(?!$)\s*', '%BEGIN\n', work)
        work = re.sub('\s*(?<!^)END%', '\nEND%', work)
        work = re.sub('END%(?!$)\s*', 'END%\n', work)

        # make each maximal term (period-terminated) start on a new line

        work = re.sub('\.\s*(?=\S)', '.\n', work)

        # Undo the any protections installed above.
        for (rep, pat) in pairs:
            work = re.sub(pat,rep,work)

        # Restore the coment (if any).

        work = work + comment  # comment may be ''

        result = result + work.splitlines(True)

    return result

def partition(input):
    assumptions = []
    goals = []
    other = []
    lines = normalize_input(input);
    state = 'other'
    for line in lines:
        if state == 'assumptions':
            if re.search('end_of_list\.', line):
                state = 'other'
            else:
                assumptions.append(line)
        elif state == 'goals':
            if re.search('end_of_list\.', line):
                state = 'other'
            else:
                goals.append(line)
        else:
            if re.search('formulas\s*\(\s*(assumptions|sos)', line):
                state = 'assumptions'
            elif re.search('formulas\s*\(\s*goals', line):
                state = 'goals'
            else:
                other.append(line)

    return (''.join(assumptions),
            ''.join(goals),
            ''.join(other))
                                       
if __name__ == '__main__':

    input = sys.stdin.read()

    (a,g,o) = partition_input(input)
    print '% other'
    for line in o:
        print line

    print '\n% Assumptions'
    print 'formulas(assumptions).'
    for line in a:
        print line
    print 'end_of_list.'

    print '\n% Goals'
    print 'formulas(goals).'
    for line in g:
        print line
    print 'end_of_list.'

